@@ -670,6 +670,71 @@ list_claude_batches <- function(.api_url = "https://api.anthropic.com/",
   return(batch_list)
 }
 
+#' List Available Models from the Anthropic Claude API
+#'
+#' @param .api_url Base URL for the API (default: "https://api.anthropic.com").
+#' @param .timeout Request timeout in seconds (default: 60).
+#' @param .max_tries Maximum number of retries for the API request (default: 3).
+#' @param .dry_run Logical; if TRUE, returns the prepared request object without executing it.
+#' @param .verbose Logical; if TRUE, prints additional information about the request.
+#'
+#' @return A tibble containing model information (columns include `type`,`id`, `display_name`, and `created_at`),
+#'   or NULL if no models are found.
+#'
+#' @export
+claude_list_models <- function(.api_url = "https://api.anthropic.com",
+                               .timeout = 60,
+                               .max_tries = 3,
+                               .dry_run = FALSE,
+                               .verbose = FALSE) {
+
+  # Create an API object for Claude using the tidyllm helper
+  api_obj <- api_claude(short_name = "claude",
+                        long_name  = "Anthropic Claude",
+                        api_key_env_var = "ANTHROPIC_API_KEY")
+  
+  # Retrieve the API key (will error if not set, unless in dry run mode)
+  api_key <- get_api_key(api_obj, .dry_run)
+  
+  # Build the request to the /v1/models endpoint
+  request <- httr2::request(.api_url) |>
+    httr2::req_url_path("/v1/models") |>
+    httr2::req_headers(
+      `x-api-key` = api_key,
+      `anthropic-version` = "2023-06-01",
+      `content-type` = "application/json; charset=utf-8"
+    )
+  
+  # If dry run is requested, return the constructed request object
+  if (.dry_run) {
+    return(request)
+  }
+  
+  # Perform the request with specified timeout and retry logic
+  response <- request |>
+    httr2::req_timeout(.timeout) |>
+    httr2::req_retry(max_tries = .max_tries) |>
+    httr2::req_perform() |>
+    httr2::resp_body_json()
+  
+  if (.verbose) {
+    message("Retrieved response from Anthropic Claude: ", response$object)
+  }
+  
+  # Check if the "data" field exists and contains models
+  if (!is.null(response$data)) {
+    models <- response$data
+    
+    # Create a tibble with selected model information
+    model_info <- models |> 
+      purrr::map_dfr(tibble::as_tibble)
+    
+    return(model_info)
+  } else {
+    return(NULL)
+  }
+}
+
 #' Provider Function for Claude models on the Anthropic API
 #'
 #' The `claude()` function acts as an interface for interacting with the Anthropic API 
@@ -693,7 +758,8 @@ claude <- create_provider_function(
   send_batch = send_claude_batch,
   check_batch = check_claude_batch,
   list_batches = list_claude_batches,
-  fetch_batch = fetch_claude_batch
+  fetch_batch = fetch_claude_batch,
+  list_models = claude_list_models
 )
 
 
